/**
 * Objet global définissant l'espace de nom BdfApi
 * 
 * v 0.0.0
 * 
 * @namespace BdfApi
 */
var BdfApi = {};


/**
 * Affiche le message dans la console si elle est opérationnelle.
 * 
 * @param {String} msg Message à afficher
 * @returns {undefined}
 */
BdfApi.log = function (msg) {
    if ((console) && (console.log)) {
        console.log(msg);
    }
};


/**
 * Affiche un objet de message d'erreur
 * 
 * @param {Object} commandMessage Objet du message
 * @param {Object} commandMessage.text Texte du message
 * @param {Object} commandMessage.key Clé du message (utilisé si le texte n'est pas défini) 
 * @returns {undefined}
 */
BdfApi.logCommandMessage = function (commandMessage) {
    var msg = "[" + commandMessage.type + "] - ";
    if (commandMessage.text) {
        msg += commandMessage.text;
    } else {
        msg += commandMessage.key;
    }
    BdfApi.log(msg);
};


/**
 * Indique si les données transmises par la réponse contiennent un message d'erreur
 * 
 * @param {Object} data Données transmises par la réponse
 * @returns {Boolean}
 */
BdfApi.hasErrorCommandMessage = function (data) {
    return ((data.commandMessage) && (data.commandMessage.type) && (data.commandMessage.type === "error"));
};


/**
 * Extrait l'objet de nom objectName des données transmises par la réponse
 * et appelle la fonction objectCallback avec cet objet en argument s'il n'y a
 * pas d'erreur
 * 
 * @param {Object} data Données transmises par la réponse
 * @param {string} objectName Nom de l'objet
 * @param {function} objectCallback fonction de rappel appelée avec l'objet extrait en argument
 * @returns {undefined}
 */
BdfApi.extractObject = function (data, objectName, objectCallback) {
    if (BdfApi.hasErrorCommandMessage(data)) {
        BdfApi.logCommandMessage(data.commandMessage);
    } else {
        if (!data.hasOwnProperty(objectName)) {
            $.error(objectName + " object is missing in json response");
        } else {
            if (data.commandMessage) {
                BdfApi.logCommandMessage(data.commandMessage);
            }
            objectCallback(data[objectName]);
        }
    }
};


/**
 * Construit l'objet destiné à la propriété data d'une requête Ajax.
 * Si l'objet options existe, rajoute les propriétés de la propriété requestParameters de l'objet options
 * 
 * @param {String} jsonName
 * @param {Object} options Objet définissant les options
 * @param {Object} options.requestParameters Surcharge des paramètres de la requête
 * @param {Object} defaultRequestParameters Paramètres par défaut de la requête
 * @returns {Object} Objet construit
 */
BdfApi.initAjaxRequestData = function (jsonName, options, defaultRequestParameters) {
    var ajaxRequestData = {};
	if (defaultRequestParameters) {
		Object.assign(ajaxRequestData, defaultRequestParameters);
	}
    if (options && options.requestParameters) {
        Object.assign(ajaxRequestData, options.requestParameters);
    }
    ajaxRequestData.json = jsonName;
    return ajaxRequestData;
};


/**
 * Traite les données de la réponse Ajax
 * 
 * @param {Object} data Données transmises par la réponse
 * @param {Object} options Options du traitement de la réponse
 * @param {function} options.callback Fonction de rappel appliquée aux données de la réponse
 * @param {function} options.ignoreLogError Ne pas tenir compte des erreurs (surcharge logError)
 * @param {boolean} logError Ne pas tenir compte des erreurs par défaut
 * @returns {undefined}
 */
BdfApi.handleResponseData = function (data, options, logError) {
  if (!options) {
      return;
  }
  if (!options.callback) {
      return;
  }
  if (logError && (!options.ignoreLogError) && (BdfApi.hasErrorCommandMessage(data))) {
      BdfApi.logCommandMessage(data.commandMessage);
      return;
  }
  options.callback(data);
};
